@@ -304,7 +304,7 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
             logging.set_verbosity(logging.WARNING)
             # Work in tmp directory - will store manifest file there
             with tempfile.TemporaryDirectory() as tmpdir:
-                with open(os.path.join(tmpdir, 'manifest.json'), 'w') as fp:
+                with open(os.path.join(tmpdir, 'manifest.json'), 'w', encoding='utf-8') as fp:
                     for audio_file in paths2audio_files:
                         label = 0.0 if self.is_regression_task else self.cfg.labels[0]
                         entry = {'audio_filepath': audio_file, 'duration': 100000.0, 'label': label}
