@@ -22,7 +22,7 @@ import torch
 from omegaconf import DictConfig
 from omegaconf.omegaconf import open_dict
 from pytorch_lightning import Trainer
-from torch.utils.data import ChainDataset
+from tqdm import tqdm
 
 from nemo.collections.asr.data.audio_to_label import AudioToSpeechLabelDataset
 from nemo.collections.asr.data.audio_to_label_dataset import get_tarred_speech_label_dataset
@@ -172,13 +172,10 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
                 normalize_audio=config.get('normalize_audio', False),
             )
 
-        if type(dataset) is ChainDataset:
-            collate_ds = dataset.datasets[0]
+        if hasattr(dataset, 'fixed_seq_collate_fn'):
+            collate_fn = dataset.fixed_seq_collate_fn
         else:
-            collate_ds = dataset
-
-        # self.labels = collate_ds.labels
-        collate_fn = collate_ds.fixed_seq_collate_fn
+            collate_fn = dataset.datasets[0].fixed_seq_collate_fn
 
         batch_size = config['batch_size']
         return torch.utils.data.DataLoader(
@@ -232,12 +229,17 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             "embs": NeuralType(('B', 'D'), AcousticEncodedRepresentation()),
         }
 
+    @typecheck()
+    def forward_for_export(self, processed_signal, processed_signal_len):
+        encoded, length = self.encoder(audio_signal=processed_signal, length=processed_signal_len)
+        logits, embs = self.decoder(encoder_output=encoded, length=length)
+        return logits, embs
+
     @typecheck()
     def forward(self, input_signal, input_signal_length):
         processed_signal, processed_signal_len = self.preprocessor(
             input_signal=input_signal, length=input_signal_length,
         )
-
         encoded, length = self.encoder(audio_signal=processed_signal, length=processed_signal_len)
         logits, embs = self.decoder(encoder_output=encoded, length=length)
         return logits, embs
@@ -432,7 +434,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         embs2 = self.get_embedding(path2audio_file2).squeeze()
         # Length Normalize
         X = embs1 / torch.linalg.norm(embs1)
-        Y = embs1 / torch.linalg.norm(embs2)
+        Y = embs2 / torch.linalg.norm(embs2)
         # Score
         similarity_score = torch.dot(X, Y) / ((torch.dot(X, X) * torch.dot(Y, Y)) ** 0.5)
         similarity_score = (similarity_score + 1) / 2
@@ -443,3 +445,36 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         else:
             logging.info(" two audio files are from different speakers")
             return False
+
+    @staticmethod
+    @torch.no_grad()
+    def get_batch_embeddings(speaker_model, manifest_filepath, batch_size=32, sample_rate=16000, device='cuda'):
+
+        speaker_model.eval()
+        if device == 'cuda':
+            speaker_model.to(device)
+
+        featurizer = WaveformFeaturizer(sample_rate=sample_rate)
+        dataset = AudioToSpeechLabelDataset(manifest_filepath=manifest_filepath, labels=None, featurizer=featurizer)
+
+        dataloader = torch.utils.data.DataLoader(
+            dataset=dataset, batch_size=batch_size, collate_fn=dataset.fixed_seq_collate_fn,
+        )
+
+        all_logits = []
+        all_labels = []
+        all_embs = []
+
+        for test_batch in tqdm(dataloader):
+            if device == 'cuda':
+                test_batch = [x.to(device) for x in test_batch]
+            audio_signal, audio_signal_len, labels, _ = test_batch
+            logits, embs = speaker_model.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
+
+            all_logits.extend(logits.cpu().numpy())
+            all_labels.extend(labels.cpu().numpy())
+            all_embs.extend(embs.cpu().numpy())
+
+        all_logits, true_labels, all_embs = np.asarray(all_logits), np.asarray(all_labels), np.asarray(all_embs)
+
+        return all_embs, all_logits, true_labels, dataset.id2label
