@@ -281,11 +281,11 @@ class GreedyRNNTInfer(_GreedyRNNTInfer):
         # out_len: [seq_len]
 
         # Initialize blank state and empty label set in Hypothesis
-        hypothesis = rnnt_utils.Hypothesis(score=0.0, y_sequence=[], dec_state=None, timestep=[])
+        hypothesis = rnnt_utils.Hypothesis(score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None)
 
         if partial_hypotheses is not None:
-            if len(partial_hypotheses.y_sequence) > 0:
-                hypothesis.y_sequence.append(partial_hypotheses.y_sequence[-1].cpu().numpy())
+            hypothesis.last_token = partial_hypotheses.last_token
+            if partial_hypotheses.dec_state is not None:
                 hypothesis.dec_state = self.decoder.batch_concat_states([partial_hypotheses.dec_state])
                 hypothesis.dec_state = _states_to_device(hypothesis.dec_state, x.device)
 
@@ -308,11 +308,10 @@ class GreedyRNNTInfer(_GreedyRNNTInfer):
             while not_blank and (self.max_symbols is None or symbols_added < self.max_symbols):
                 # In the first timestep, we initialize the network with RNNT Blank
                 # In later timesteps, we provide previous predicted label as input.
-                last_label = (
-                    self._SOS
-                    if (hypothesis.y_sequence == [] and hypothesis.dec_state is None)
-                    else hypothesis.y_sequence[-1]
-                )
+                if hypothesis.last_token is None and hypothesis.dec_state is None:
+                    last_label = self._SOS
+                else:
+                    last_label = label_collate([[hypothesis.last_token]])
 
                 # Perform prediction network and joint network steps.
                 g, hidden_prime = self._pred_step(last_label, hypothesis.dec_state)
@@ -347,6 +346,7 @@ class GreedyRNNTInfer(_GreedyRNNTInfer):
                     hypothesis.score += float(v)
                     hypothesis.timestep.append(time_idx)
                     hypothesis.dec_state = hidden_prime
+                    hypothesis.last_token = k
 
                 # Increment token counter.
                 symbols_added += 1
@@ -359,10 +359,6 @@ class GreedyRNNTInfer(_GreedyRNNTInfer):
         # Unpack the hidden states
         hypothesis.dec_state = self.decoder.batch_select_state(hypothesis.dec_state, 0)
 
-        # Remove the original input label if partial hypothesis was provided
-        if partial_hypotheses is not None:
-            hypothesis.y_sequence = hypothesis.y_sequence[1:]
-
         return hypothesis
 
 
@@ -464,7 +460,7 @@ class GreedyBatchedRNNTInfer(_GreedyRNNTInfer):
         if partial_hypotheses is not None:
             raise NotImplementedError("`partial_hypotheses` support is not supported")
 
-        with torch.no_grad():
+        with torch.inference_mode():
             # x: [B, T, D]
             # out_len: [B]
             # device: torch.device
@@ -512,9 +508,9 @@ class GreedyBatchedRNNTInfer(_GreedyRNNTInfer):
                 # Batch: [B, T, D], but Bi may have seq len < max(seq_lens_in_batch)
                 # Forcibly mask with "blank" tokens, for all sample where current time step T > seq_len
                 blank_mask = time_idx >= out_len
+
                 # Start inner loop
                 while not_blank and (self.max_symbols is None or symbols_added < self.max_symbols):
-
                     # Batch prediction and joint network steps
                     # If very first prediction step, submit SOS tag (blank) to pred_step.
                     # This feeds a zero tensor as input to AbstractRNNTDecoder to prime the state
@@ -619,7 +615,6 @@ class GreedyBatchedRNNTInfer(_GreedyRNNTInfer):
 
         return hypotheses
 
-    @torch.no_grad()
     def _greedy_decode_masked(
         self,
         x: torch.Tensor,
@@ -661,121 +656,123 @@ class GreedyBatchedRNNTInfer(_GreedyRNNTInfer):
 
         # Get max sequence length
         max_out_len = out_len.max()
-        for time_idx in range(max_out_len):
-            f = x.narrow(dim=1, start=time_idx, length=1)  # [B, 1, D]
 
-            # Prepare t timestamp batch variables
-            not_blank = True
-            symbols_added = 0
+        with torch.inference_mode():
+            for time_idx in range(max_out_len):
+                f = x.narrow(dim=1, start=time_idx, length=1)  # [B, 1, D]
 
-            # Reset blank mask
-            blank_mask.mul_(False)
+                # Prepare t timestamp batch variables
+                not_blank = True
+                symbols_added = 0
 
-            # Update blank mask with time mask
-            # Batch: [B, T, D], but Bi may have seq len < max(seq_lens_in_batch)
-            # Forcibly mask with "blank" tokens, for all sample where current time step T > seq_len
-            blank_mask = time_idx >= out_len
+                # Reset blank mask
+                blank_mask.mul_(False)
 
-            # Start inner loop
-            while not_blank and (self.max_symbols is None or symbols_added < self.max_symbols):
-                # Batch prediction and joint network steps
-                # If very first prediction step, submit SOS tag (blank) to pred_step.
-                # This feeds a zero tensor as input to AbstractRNNTDecoder to prime the state
-                if time_idx == 0 and symbols_added == 0 and hidden is None:
-                    g, hidden_prime = self._pred_step(self._SOS, hidden, batch_size=batchsize)
-                else:
-                    # Set a dummy label for the blank value
-                    # This value will be overwritten by "blank" again the last label update below
-                    # This is done as vocabulary of prediction network does not contain "blank" token of RNNT
-                    last_label_without_blank_mask = last_label == self._blank_index
-                    last_label_without_blank[last_label_without_blank_mask] = 0  # temp change of label
-                    last_label_without_blank[~last_label_without_blank_mask] = last_label[
-                        ~last_label_without_blank_mask
-                    ]
+                # Update blank mask with time mask
+                # Batch: [B, T, D], but Bi may have seq len < max(seq_lens_in_batch)
+                # Forcibly mask with "blank" tokens, for all sample where current time step T > seq_len
+                blank_mask = time_idx >= out_len
 
-                    # Perform batch step prediction of decoder, getting new states and scores ("g")
-                    g, hidden_prime = self._pred_step(last_label_without_blank, hidden, batch_size=batchsize)
+                # Start inner loop
+                while not_blank and (self.max_symbols is None or symbols_added < self.max_symbols):
+                    # Batch prediction and joint network steps
+                    # If very first prediction step, submit SOS tag (blank) to pred_step.
+                    # This feeds a zero tensor as input to AbstractRNNTDecoder to prime the state
+                    if time_idx == 0 and symbols_added == 0 and hidden is None:
+                        g, hidden_prime = self._pred_step(self._SOS, hidden, batch_size=batchsize)
+                    else:
+                        # Set a dummy label for the blank value
+                        # This value will be overwritten by "blank" again the last label update below
+                        # This is done as vocabulary of prediction network does not contain "blank" token of RNNT
+                        last_label_without_blank_mask = last_label == self._blank_index
+                        last_label_without_blank[last_label_without_blank_mask] = 0  # temp change of label
+                        last_label_without_blank[~last_label_without_blank_mask] = last_label[
+                            ~last_label_without_blank_mask
+                        ]
 
-                # Batched joint step - Output = [B, V + 1]
-                logp = self._joint_step(f, g, log_normalize=None)[:, 0, 0, :]
+                        # Perform batch step prediction of decoder, getting new states and scores ("g")
+                        g, hidden_prime = self._pred_step(last_label_without_blank, hidden, batch_size=batchsize)
 
-                if logp.dtype != torch.float32:
-                    logp = logp.float()
+                    # Batched joint step - Output = [B, V + 1]
+                    logp = self._joint_step(f, g, log_normalize=None)[:, 0, 0, :]
 
-                # Get index k, of max prob for batch
-                v, k = logp.max(1)
-                del g
+                    if logp.dtype != torch.float32:
+                        logp = logp.float()
 
-                # Update blank mask with current predicted blanks
-                # This is accumulating blanks over all time steps T and all target steps min(max_symbols, U)
-                k_is_blank = k == self._blank_index
-                blank_mask.bitwise_or_(k_is_blank)
+                    # Get index k, of max prob for batch
+                    v, k = logp.max(1)
+                    del g
 
-                # If preserving alignments, check if sequence length of sample has been reached
-                # before adding alignment
-                if self.preserve_alignments:
-                    # Insert ids into last timestep per sample
-                    logp_vals = logp.to('cpu').max(1)[1]
-                    for batch_idx in range(batchsize):
-                        if time_idx < out_len[batch_idx]:
-                            hypotheses[batch_idx].alignments[-1].append(logp_vals[batch_idx])
-                    del logp_vals
-                del logp
+                    # Update blank mask with current predicted blanks
+                    # This is accumulating blanks over all time steps T and all target steps min(max_symbols, U)
+                    k_is_blank = k == self._blank_index
+                    blank_mask.bitwise_or_(k_is_blank)
 
-                # If all samples predict / have predicted prior blanks, exit loop early
-                # This is equivalent to if single sample predicted k
-                if blank_mask.all():
-                    not_blank = False
-
-                    # If preserving alignments, convert the current Uj alignments into a torch.Tensor
-                    # Then preserve U at current timestep Ti
-                    # Finally, forward the timestep history to Ti+1 for that sample
-                    # All of this should only be done iff the current time index <= sample-level AM length.
-                    # Otherwise ignore and move to next sample / next timestep.
+                    # If preserving alignments, check if sequence length of sample has been reached
+                    # before adding alignment
                     if self.preserve_alignments:
-
-                        # convert Ti-th logits into a torch array
+                        # Insert ids into last timestep per sample
+                        logp_vals = logp.to('cpu').max(1)[1]
                         for batch_idx in range(batchsize):
+                            if time_idx < out_len[batch_idx]:
+                                hypotheses[batch_idx].alignments[-1].append(logp_vals[batch_idx])
+                        del logp_vals
+                    del logp
 
-                            # this checks if current timestep <= sample-level AM length
-                            # If current timestep > sample-level AM length, no alignments will be added
-                            # Therefore the list of Uj alignments is empty here.
-                            if len(hypotheses[batch_idx].alignments[-1]) > 0:
-                                hypotheses[batch_idx].alignments.append([])  # blank buffer for next timestep
-                else:
-                    # Collect batch indices where blanks occurred now/past
-                    blank_indices = (blank_mask == 1).nonzero(as_tuple=False)
+                    # If all samples predict / have predicted prior blanks, exit loop early
+                    # This is equivalent to if single sample predicted k
+                    if blank_mask.all():
+                        not_blank = False
 
-                    # Recover prior state for all samples which predicted blank now/past
-                    if hidden is not None:
-                        # LSTM has 2 states
-                        hidden_prime = self.decoder.batch_copy_states(hidden_prime, hidden, blank_indices)
+                        # If preserving alignments, convert the current Uj alignments into a torch.Tensor
+                        # Then preserve U at current timestep Ti
+                        # Finally, forward the timestep history to Ti+1 for that sample
+                        # All of this should only be done iff the current time index <= sample-level AM length.
+                        # Otherwise ignore and move to next sample / next timestep.
+                        if self.preserve_alignments:
 
-                    elif len(blank_indices) > 0 and hidden is None:
-                        # Reset state if there were some blank and other non-blank predictions in batch
-                        # Original state is filled with zeros so we just multiply
-                        # LSTM has 2 states
-                        hidden_prime = self.decoder.batch_copy_states(hidden_prime, None, blank_indices, value=0.0)
+                            # convert Ti-th logits into a torch array
+                            for batch_idx in range(batchsize):
 
-                    # Recover prior predicted label for all samples which predicted blank now/past
-                    k[blank_indices] = last_label[blank_indices, 0]
+                                # this checks if current timestep <= sample-level AM length
+                                # If current timestep > sample-level AM length, no alignments will be added
+                                # Therefore the list of Uj alignments is empty here.
+                                if len(hypotheses[batch_idx].alignments[-1]) > 0:
+                                    hypotheses[batch_idx].alignments.append([])  # blank buffer for next timestep
+                    else:
+                        # Collect batch indices where blanks occurred now/past
+                        blank_indices = (blank_mask == 1).nonzero(as_tuple=False)
 
-                    # Update new label and hidden state for next iteration
-                    last_label = k.view(-1, 1)
-                    hidden = hidden_prime
+                        # Recover prior state for all samples which predicted blank now/past
+                        if hidden is not None:
+                            # LSTM has 2 states
+                            hidden_prime = self.decoder.batch_copy_states(hidden_prime, hidden, blank_indices)
 
-                    # Update predicted labels, accounting for time mask
-                    # If blank was predicted even once, now or in the past,
-                    # Force the current predicted label to also be blank
-                    # This ensures that blanks propogate across all timesteps
-                    # once they have occured (normally stopping condition of sample level loop).
-                    for kidx, ki in enumerate(k):
-                        if blank_mask[kidx] == 0:
-                            hypotheses[kidx].y_sequence.append(ki)
-                            hypotheses[kidx].timestep.append(time_idx)
-                            hypotheses[kidx].score += float(v[kidx])
+                        elif len(blank_indices) > 0 and hidden is None:
+                            # Reset state if there were some blank and other non-blank predictions in batch
+                            # Original state is filled with zeros so we just multiply
+                            # LSTM has 2 states
+                            hidden_prime = self.decoder.batch_copy_states(hidden_prime, None, blank_indices, value=0.0)
 
-                symbols_added += 1
+                        # Recover prior predicted label for all samples which predicted blank now/past
+                        k[blank_indices] = last_label[blank_indices, 0]
+
+                        # Update new label and hidden state for next iteration
+                        last_label = k.view(-1, 1)
+                        hidden = hidden_prime
+
+                        # Update predicted labels, accounting for time mask
+                        # If blank was predicted even once, now or in the past,
+                        # Force the current predicted label to also be blank
+                        # This ensures that blanks propogate across all timesteps
+                        # once they have occured (normally stopping condition of sample level loop).
+                        for kidx, ki in enumerate(k):
+                            if blank_mask[kidx] == 0:
+                                hypotheses[kidx].y_sequence.append(ki)
+                                hypotheses[kidx].timestep.append(time_idx)
+                                hypotheses[kidx].score += float(v[kidx])
+
+                    symbols_added += 1
 
         # Remove trailing empty list of alignments at T_{am-len} x Uj
         if self.preserve_alignments:
