@@ -38,11 +38,17 @@ class T5Dataset(MegatronDataset):
         data_prefix,
         num_epochs,
         max_num_samples,
-        masked_lm_prob,
         max_seq_length,
         max_seq_length_dec,
-        short_seq_prob,
         seed,
+        masked_lm_prob=0.15,
+        short_seq_prob=0.1,
+        max_ngram_size=10,
+        mean_ngram_size=None,
+        geometric_dist=True,
+        permutation=False,
+        whole_word_masking=True,
+        favor_long_ngrams=False,
     ):
         super().__init__(cfg, trainer=trainer)
 
@@ -52,31 +58,43 @@ class T5Dataset(MegatronDataset):
         self.masked_lm_prob = masked_lm_prob
         self.max_seq_length = max_seq_length
         self.max_seq_length_dec = max_seq_length_dec
+        self.short_seq_prob = short_seq_prob
+        self.max_ngram_size = max_ngram_size
+        self.mean_ngram_size = mean_ngram_size
+        self.geometric_dist = geometric_dist
+        self.permutation = permutation
+        self.whole_word_masking = whole_word_masking
+        self.favor_long_ngrams = favor_long_ngrams
 
         # Dataset.
         self.indexed_dataset = indexed_dataset
 
         # Build the samples mapping.
         self.samples_mapping = get_samples_mapping(
-            self.indexed_dataset,
-            data_prefix,
-            num_epochs,
-            max_num_samples,
-            self.max_seq_length - 2,  # account for added tokens
-            short_seq_prob,
-            self.seed,
-            self.name,
-            False,
+            indexed_dataset=self.indexed_dataset,
+            data_prefix=data_prefix,
+            num_epochs=num_epochs,
+            max_num_samples=max_num_samples,
+            max_seq_length=self.max_seq_length - 2,  # account for added tokens
+            short_seq_prob=self.short_seq_prob,
+            seed=self.seed,
+            name=self.name,
+            binary_head=False,
         )
 
         self.tokenizer = tokenizer
-
+        self.tokenizer_type = 'wordpiece'  # TODO: better checks for tokenizer types. How do we do this for HF tokenizers that are not BERT?
         if isinstance(self.tokenizer, YouTokenToMeTokenizer):
             raise ValueError(f"YTTM does not support special tokens and cannot be used with T5 datasets.")
 
         if isinstance(self.tokenizer, SentencePieceTokenizer):
             if not self.tokenizer.legacy:
                 raise ValueError("Sentencepiece Tokenizer must have legacy = False to add special tokens.")
+            self.tokenizer_type = 'sentencepiece'
+            if whole_word_masking:
+                raise ValueError(
+                    "Whole word masking is not supported with sentencepiece tokenizers and only with wordpiece tokenizers. Please set it to False."
+                )
 
         self.cls_id = tokenizer.cls_id
         self.sep_id = tokenizer.sep_id
@@ -89,7 +107,7 @@ class T5Dataset(MegatronDataset):
         self.vocab_id_to_token_dict = {idx: token for idx, token in enumerate(self.vocab_id_list)}
 
         self.sentinel_tokens = tokenizer.additional_special_tokens_ids
-        assert len(self.sentinel_tokens) > 0, "Provide the argument --vocab-extra-ids 100 to the script"
+        assert len(self.sentinel_tokens) > 0
 
     def __len__(self):
         return self.samples_mapping.shape[0]
@@ -115,10 +133,17 @@ class T5Dataset(MegatronDataset):
             mask_id=self.mask_id,
             pad_id=self.pad_id,
             masked_lm_prob=self.masked_lm_prob,
+            max_ngram_size=self.max_ngram_size,
+            mean_ngram_size=self.mean_ngram_size,
+            geometric_dist=self.geometric_dist,
+            permutation=self.permutation,
+            whole_word_masking=self.whole_word_masking,
+            favor_long_ngrams=self.favor_long_ngrams,
             np_rng=np_rng,
             bos_id=self.bos_id,
             eos_id=self.eos_id,
             sentinel_tokens=self.sentinel_tokens,
+            tokenizer_type=self.tokenizer_type,
         )
         return training_sample
 
@@ -135,10 +160,17 @@ def build_training_sample(
     mask_id,
     pad_id,
     masked_lm_prob,
+    max_ngram_size,
+    mean_ngram_size,
+    geometric_dist,
+    permutation,
+    whole_word_masking,
+    favor_long_ngrams,
     np_rng,
     bos_id=None,
     eos_id=None,
     sentinel_tokens=None,
+    tokenizer_type="wordpiece",
 ):
     """Build training sample.
     Arguments:
@@ -159,6 +191,13 @@ def build_training_sample(
         bos_id: start of decoder example id
         eos_id: end of generation id
         sentinel_tokens: unique value to be substituted for every replaced span
+        tokenizer_type: wordpiece (BERT-style) or sentencepiece tokenizer. Used for whole word masking logic.
+        max_ngram_size: maximum size of ngrams to be masked.
+        mean_ngram_size: mean size of ngrams to be masked (only used if geometric_dist=True).
+        geometric_dist: Uses a geometric distribution to sample ngram size.
+        permutation: Permutes the ngrams.
+        whole_word_masking: Always masks entire words instead of individual sub-word tokens.
+        favor_long_ngrams: Favor longer ngrams over shorter ones.
     """
     assert target_seq_length <= max_seq_length
 
@@ -173,22 +212,27 @@ def build_training_sample(
     # Masking.
     max_predictions_per_seq = masked_lm_prob * max_num_tokens
     (tokens, masked_positions, masked_labels, _, masked_spans) = create_masked_lm_predictions(
-        tokens,
-        vocab_id_list,
-        vocab_id_to_token_dict,
-        masked_lm_prob,
-        cls_id,
-        sep_id,
-        mask_id,
-        max_predictions_per_seq,
-        np_rng,
-        max_ngrams=10,
-        geometric_dist=True,
+        tokens=tokens,
+        vocab_id_list=vocab_id_list,
+        vocab_id_to_token_dict=vocab_id_to_token_dict,
+        masked_lm_prob=masked_lm_prob,
+        cls_id=cls_id,
+        sep_id=sep_id,
+        mask_id=mask_id,
+        max_predictions_per_seq=max_predictions_per_seq,
+        np_rng=np_rng,
+        max_ngram_size=max_ngram_size,
+        whole_word_masking=whole_word_masking,
+        favor_long_ngrams=favor_long_ngrams,
+        mean_ngram_size=mean_ngram_size,
+        permutation=permutation,
+        geometric_dist=geometric_dist,
         masking_style="t5",
+        tokenizer_type=tokenizer_type,
     )
 
     # Padding.
-    tokens_enc, tokens_dec_in, labels, enc_mask, dec_mask, enc_dec_mask, loss_mask = pad_and_convert_to_numpy(
+    tokens_enc, tokens_dec_in, labels, enc_mask, dec_mask, loss_mask = pad_and_convert_to_numpy(
         tokens,
         masked_positions,
         masked_labels,
@@ -209,7 +253,6 @@ def build_training_sample(
         'truncated': int(truncated),
         'enc_mask': enc_mask,
         'dec_mask': dec_mask,
-        'enc_dec_mask': enc_dec_mask,
     }
     return train_sample
 
@@ -276,10 +319,8 @@ def pad_and_convert_to_numpy(
     tokens_dec_in = np.array(t5_decoder_in + filler_dec, dtype=np.int64)
 
     # Create attention masks
-    enc_mask = make_attention_mask(tokens_enc, tokens_enc, pad_id)
-    enc_dec_mask = make_attention_mask(tokens_dec_in, tokens_enc, pad_id)
-    dec_mask = make_attention_mask(tokens_dec_in, tokens_dec_in, pad_id)
-    dec_mask = dec_mask * make_history_mask(tokens_dec_in)
+    enc_mask = (tokens_enc != pad_id).astype(np.int64)
+    dec_mask = (tokens_dec_in != pad_id).astype(np.int64)
 
     # Labels mask.
     labels = t5_decoder_out + ([-1] * padding_length_dec)
@@ -289,7 +330,7 @@ def pad_and_convert_to_numpy(
     loss_mask = ([1] * num_tokens_dec) + ([0] * padding_length_dec)
     loss_mask = np.array(loss_mask, dtype=np.int64)
 
-    return tokens_enc, tokens_dec_in, labels, enc_mask, dec_mask, enc_dec_mask, loss_mask
+    return tokens_enc, tokens_dec_in, labels, enc_mask, dec_mask, loss_mask
 
 
 def make_attention_mask(source_block, target_block, pad_id):
