@@ -1,4 +1,4 @@
-# Copyright (c) 2021, NVIDIA CORPORATION.  All rights reserved.
+# Copyright (c) 2022, NVIDIA CORPORATION.  All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,34 +17,28 @@ import math
 
 import torch
 import torch.nn as nn
-import torch.nn.functional as F
 import torch.nn.init as init
-from apex.transformer import parallel_state, tensor_parallel
-from apex.transformer.enums import AttnMaskType, LayerType
 
 from nemo.collections.nlp.modules.common.megatron.module import MegatronModule
 from nemo.collections.nlp.modules.common.megatron.transformer import ParallelTransformer
 from nemo.collections.nlp.modules.common.megatron.utils import (
+    ApexGuardDefaults,
     get_linear_layer,
     init_method_normal,
     scaled_init_method_normal,
 )
 
+try:
+    from apex.transformer import parallel_state, tensor_parallel
+    from apex.transformer.enums import AttnMaskType, LayerType
 
-def parallel_lm_logits(input_, word_embeddings_weight, parallel_output, bias=None):
-    """LM logits using word embedding weights."""
-    # Parallel logits.
-    input_parallel = tensor_parallel.copy_to_tensor_model_parallel_region(input_)
-    # Matrix multiply.
-    if bias is None:
-        logits_parallel = F.linear(input_parallel, word_embeddings_weight)
-    else:
-        logits_parallel = F.linear(input_parallel, word_embeddings_weight, bias)
-    # Gather if needed.
-    if parallel_output:
-        return logits_parallel
+    HAVE_APEX = True
+except (ImportError, ModuleNotFoundError):
+    HAVE_APEX = False
 
-    return tensor_parallel.gather_from_tensor_model_parallel_region(logits_parallel)
+    # fake missing classes with None attributes
+    AttnMaskType = ApexGuardDefaults()
+    LayerType = ApexGuardDefaults()
 
 
 def get_language_model(
@@ -74,12 +68,13 @@ def get_language_model(
     activations_checkpoint_num_layers=1,
     layernorm_epsilon=1e-5,
     bias_gelu_fusion=True,
+    masked_softmax_fusion=True,
     persist_layer_norm=False,
     openai_gelu=False,
     onnx_safe=False,
     use_soft_prompts=False,
     num_prompt_tokens=10,
-    prompt_tags=None,
+    existing_prompt_tags=None,
 ):
     """Build language model and return along with the key to save."""
 
@@ -122,12 +117,13 @@ def get_language_model(
         activations_checkpoint_num_layers=activations_checkpoint_num_layers,
         layernorm_epsilon=layernorm_epsilon,
         bias_gelu_fusion=bias_gelu_fusion,
+        masked_softmax_fusion=masked_softmax_fusion,
         persist_layer_norm=persist_layer_norm,
         openai_gelu=openai_gelu,
         onnx_safe=onnx_safe,
         use_soft_prompts=use_soft_prompts,
         num_prompt_tokens=num_prompt_tokens,
-        prompt_tags=prompt_tags,
+        existing_prompt_tags=existing_prompt_tags,
     )
     # key used for checkpoints.
     language_model_key = 'language_model'
@@ -231,10 +227,15 @@ class Embedding(MegatronModule):
         # Initialize the token-type embeddings.
         self.init_method(self.tokentype_embeddings.weight)
 
-    def forward(self, input_ids, position_ids, tokentype_ids=None):
+    def forward(self, input_ids, position_ids, tokentype_ids=None, separate_embeddings=False):
         # Embeddings.
         words_embeddings = self.word_embeddings(input_ids)
         position_embeddings = self.position_embeddings(position_ids)
+
+        # Want word embeddings and position embeddings before addition for soft prompt initalization
+        if separate_embeddings:
+            return words_embeddings, position_embeddings
+
         embeddings = words_embeddings + position_embeddings
         if tokentype_ids is not None:
             assert self.tokentype_embeddings is not None
@@ -337,30 +338,36 @@ class PromptEmbedding(MegatronModule):
 
         self.hidden_size = hidden_size
         self.num_prompt_tokens = num_prompt_tokens
+
+        # Randomly init token and position embeddings
         self.prompt_embeddings = torch.nn.Embedding(self.num_prompt_tokens, self.hidden_size)
-        init_method(self.prompt_embeddings.weight)
-
-        if init_from_prompt_text:
-
-            # Set embedding weights to be embeddings from prompt tokens
-            self.prompt_embeddings.weight = nn.Parameter(word_embedding_weights)
-
-        self._prompt_embeddings_key = 'prompt_embeddings'
-
         self.position_embeddings = torch.nn.Embedding(self.num_prompt_tokens, self.hidden_size)
-        self._position_embeddings_key = 'position_embeddings'
+        init_method(self.prompt_embeddings.weight)
+        init_method(self.position_embeddings.weight)
 
+        # Set embedding weights to be embeddings from prompt tokens
+        if init_from_prompt_text:
+            self.prompt_embeddings.weight = nn.Parameter(word_embedding_weights)
         if position_embedding_weights != None:
             self.position_embeddings.weight = nn.Parameter(position_embedding_weights)
 
-        self.prompt_ids = torch.tensor([i for i in range(self.num_prompt_tokens)])
+        # Set keys for loading and saving weights
+        self._prompt_embeddings_key = 'prompt_embeddings'
+        self._position_embeddings_key = 'position_embeddings'
+
+        # Set ids needed for forward pass and broadcast them
+        # ids = {'ids': torch.arange(self.num_prompt_tokens, dtype=torch.int64)}
+        # ids_b = tensor_parallel.broadcast_data(['ids'], ids, torch.int64)
+        # self.ids = ids_b['ids'].long()
+        self.ids = torch.arange(self.num_prompt_tokens, dtype=torch.int64)
+
         self.embedding_dropout = torch.nn.Dropout(prompt_embedding_dropout_prob)
 
     def forward(self, tokentype_ids=None):
         # Embeddings.
         device = next(self.prompt_embeddings.parameters()).device
-        prompt_embeddings = self.prompt_embeddings(self.prompt_ids.to(device))
-        position_embeddings = self.position_embeddings(self.prompt_ids.to(device))
+        prompt_embeddings = self.prompt_embeddings(self.ids.to(device))
+        position_embeddings = self.position_embeddings(self.ids.to(device))
         embeddings = prompt_embeddings + position_embeddings
 
         # Dropout.
@@ -408,50 +415,57 @@ class PromptEmbedding(MegatronModule):
 
 class PromptTable(torch.nn.Module):
     def __init__(
-        self, prompt_tags, num_prompt_tokens, hidden_size,
+        self, existing_prompt_tags, num_prompt_tokens, hidden_size,
     ):
         super().__init__()
 
         self.num_prompt_tokens = num_prompt_tokens
         self.hidden_size = hidden_size
         self.prompt_table = torch.nn.ModuleDict()
+        self.prompt_id_to_tag = {}
 
-        if prompt_tags:
-            for tag in enumerate(prompt_tags):
-                _, tag = tag
+        if existing_prompt_tags:
+            for tag, prompt_id in existing_prompt_tags:
+                self.prompt_id_to_tag[prompt_id] = tag
                 self.prompt_table[tag] = PromptEmbedding(
                     init_from_prompt_text=False,
                     hidden_size=self.hidden_size,
                     num_prompt_tokens=self.num_prompt_tokens,
                 )
 
-    def forward(self, prompt_tag):
+    def forward(self, prompt_id):
+        prompt_id = prompt_id.item()
+        prompt_tag = self.prompt_id_to_tag[prompt_id]
         return self.prompt_table[prompt_tag]()
 
     def remove_prompt(self, prompt_tag):
+        if prompt_tag not in prompt_table:
+            return
+
+        # find the prompt_id assocaited with the tag to delete
+        prompt_id = None
+        for key, value in prompt_id_to_tag.items():
+            if value == prompt_tag:
+                prompt_id = key
+                break
+
+        del self.prompt_id_to_tag[prompt_id]
         del self.prompt_table[prompt_tag]
 
-    def init_prompt_from_random(self, prompt_tag, position_embeddings):
+    def init_prompt_from_random(self, prompt_tag, prompt_id, embeddings):
         """Add new soft prompt to be tuned.
            Intialize prompt weights using pytorch init method
 
         """
-        device = next(position_embeddings.parameters()).device
-        position_weights = (
-            position_embeddings(torch.tensor([i for i in range(self.num_prompt_tokens)]).to(device)).detach().clone()
-        )
-
         # Initalize prompt embeddings from a pytorch random init method
         prompt_embeddings = PromptEmbedding(
-            init_from_prompt_text=False,
-            hidden_size=self.hidden_size,
-            num_prompt_tokens=self.num_prompt_tokens,
-            position_embedding_weights=position_weights,
+            init_from_prompt_text=False, hidden_size=self.hidden_size, num_prompt_tokens=self.num_prompt_tokens,
         )
 
         self.prompt_table[prompt_tag] = prompt_embeddings
+        self.prompt_id_to_tag[prompt_id] = prompt_tag
 
-    def init_prompt_from_text(self, prompt_tag, init_token_ids, word_embeddings, position_embeddings):
+    def init_prompt_from_text(self, prompt_tag, prompt_id, init_token_ids, embeddings):
         """Add new soft prompt to be tuned.
            Intialize prompt weights from existing embeddings from specific vocab tokens.
 
@@ -465,26 +479,34 @@ class PromptTable(torch.nn.Module):
         elif num_text_tokens < num_prompt_tokens:
             num_reps = math.ceil(num_prompt_tokens / num_text_tokens)
             init_token_ids = init_token_ids * num_reps
+
+        # Set dictionary item keys and datatypes for broadcasting
+        keys = ['text']
+        datatype = torch.int64
+
+        # Broadcast int ids across gpus for tensor parallel
         init_token_ids = init_token_ids[:num_prompt_tokens]
+        init_token_ids = {'text': torch.tensor(init_token_ids, dtype=torch.int64)}
+        init_token_ids_b = tensor_parallel.broadcast_data(keys, init_token_ids, datatype)
+        init_token_ids = init_token_ids_b['text'].long()
+        init_position_ids = torch.arange(self.num_prompt_tokens, dtype=torch.long, device=init_token_ids.device)
 
         # Use a copy of token embedding weights to initalize the prompt embeddings
-        device = next(word_embeddings.parameters()).device
-        embedding_weights = word_embeddings(torch.tensor(init_token_ids, device=device)).detach().clone()
-        position_weights = (
-            position_embeddings(torch.tensor([i for i in range(self.num_prompt_tokens)], device=device))
-            .detach()
-            .clone()
-        )
+        word_embeddings, position_embeddings = embeddings(init_token_ids, init_position_ids, separate_embeddings=True)
+
+        word_embeddings = word_embeddings.detach().clone()
+        position_embeddings = position_embeddings.detach().clone()
 
         prompt_embeddings = PromptEmbedding(
             init_from_prompt_text=True,
             hidden_size=self.hidden_size,
             num_prompt_tokens=self.num_prompt_tokens,
-            word_embedding_weights=embedding_weights,
-            position_embedding_weights=position_weights,
+            word_embedding_weights=word_embeddings,
+            position_embedding_weights=position_embeddings,
         )
 
         self.prompt_table[prompt_tag] = prompt_embeddings
+        self.prompt_id_to_tag[prompt_id] = prompt_tag
 
     def load_state_dict(self, state_dict_, strict):
         for prompt_tag in self.prompt_table:
@@ -540,12 +562,13 @@ class TransformerLanguageModel(MegatronModule):
         activations_checkpoint_num_layers=1,
         layernorm_epsilon=1e-5,
         bias_gelu_fusion=True,
+        masked_softmax_fusion=True,
         persist_layer_norm=False,
         openai_gelu=False,
         onnx_safe=False,
         use_soft_prompts=False,
-        num_prompt_tokens=10,
-        prompt_tags=None,
+        num_prompt_tokens=100,
+        existing_prompt_tags=None,
     ):
         super(TransformerLanguageModel, self).__init__()
 
@@ -564,7 +587,7 @@ class TransformerLanguageModel(MegatronModule):
         self.hidden_dropout = hidden_dropout
         self.output_layer_init_method = output_layer_init_method
         self.use_soft_prompts = use_soft_prompts
-        self.prompt_tags = prompt_tags
+        self.existing_prompt_tags = existing_prompt_tags
         self.num_prompt_tokens = num_prompt_tokens
 
         if kv_channels is None:
@@ -590,7 +613,9 @@ class TransformerLanguageModel(MegatronModule):
         # Soft Prompts
         if self.use_soft_prompts:
             self.prompt_table = PromptTable(
-                prompt_tags=self.prompt_tags, num_prompt_tokens=self.num_prompt_tokens, hidden_size=self.hidden_size,
+                existing_prompt_tags=self.existing_prompt_tags,
+                num_prompt_tokens=self.num_prompt_tokens,
+                hidden_size=self.hidden_size,
             )
             self._prompt_table_key = 'prompt_table'
 
@@ -618,6 +643,7 @@ class TransformerLanguageModel(MegatronModule):
             persist_layer_norm=persist_layer_norm,
             openai_gelu=openai_gelu,
             onnx_safe=onnx_safe,
+            masked_softmax_fusion=masked_softmax_fusion,
         )
         self._encoder_key = 'encoder'
 
@@ -650,6 +676,7 @@ class TransformerLanguageModel(MegatronModule):
                 persist_layer_norm=persist_layer_norm,
                 openai_gelu=openai_gelu,
                 onnx_safe=onnx_safe,
+                masked_softmax_fusion=masked_softmax_fusion,
             )
             self._decoder_key = 'decoder'
 
@@ -661,14 +688,19 @@ class TransformerLanguageModel(MegatronModule):
 
     def set_input_tensor(self, input_tensor):
         """ See megatron.model.transformer.set_input_tensor()"""
-        self.encoder.set_input_tensor(input_tensor)
+        # This is usually handled in schedules.py but some inference code still
+        # gives us non-lists or None
+        if not isinstance(input_tensor, list):
+            input_tensor = [input_tensor]
+
+        self.encoder.set_input_tensor(input_tensor[0])
 
     def forward(
         self,
         enc_input_ids,
         enc_position_ids,
         enc_attn_mask,
-        prompt_tags=None,
+        prompt_ids=None,
         dec_input_ids=None,
         dec_position_ids=None,
         dec_attn_mask=None,
@@ -679,22 +711,21 @@ class TransformerLanguageModel(MegatronModule):
         pooling_sequence_index=0,
         enc_hidden_states=None,
         output_enc_hidden_only=False,
+        encoder_input=None,
     ):
         # Embeddings.
-        if self.pre_process:
+        if self.pre_process and encoder_input is None:
             embedding_output = self.embedding(enc_input_ids, enc_position_ids, tokentype_ids=tokentype_ids)
 
             # Soft prompts
-            if self.use_soft_prompts and prompt_tags:
-                prompt_embeddings = [self.prompt_table(tag) for tag in prompt_tags]
+            if self.use_soft_prompts and prompt_ids != None:
+                prompt_embeddings = [self.prompt_table(prompt_id) for prompt_id in prompt_ids]
                 prompt_embeddings = torch.stack(prompt_embeddings)
-
                 encoder_input = torch.cat((prompt_embeddings, embedding_output), dim=1)
-
             else:
                 encoder_input = embedding_output
         else:
-            encoder_input = None
+            pass
 
         # encoder.
         if enc_hidden_states is None:
@@ -818,29 +849,26 @@ class TransformerLanguageModel(MegatronModule):
             assert 'decoder' in state_dict, 'could not find data for pooler in the checkpoint'
             self.decoder.load_state_dict(state_dict[self._decoder_key], strict=strict)
 
-    def _init_prompt_from_random(self, prompt_tag):
+    def _init_prompt_from_random(self, prompt_tag, prompt_id):
         """Add new soft prompt to be tuned.
            Intialize prompt weights using pytorch init method
 
         """
+        if self.pre_process:
+            if not hasattr(self, 'prompt_table'):
+                raise AttributeError('Please set "use_soft_prompts" in the config to True')
 
-        if not hasattr(self, 'prompt_table'):
-            raise AttributeError('Please set "use_soft_prompts" in the config to True')
+            self.prompt_table.init_prompt_from_random(prompt_tag, prompt_id, embeddings=self.embedding)
 
-        self.prompt_table.init_prompt_from_random(prompt_tag, self.embedding.position_embeddings)
-
-    def _init_prompt_from_text(self, prompt_tag, init_token_ids):
+    def _init_prompt_from_text(self, prompt_tag, prompt_id, init_token_ids):
         """Add new soft prompt to be tuned.
            Intialize prompt weights from existing embeddings from specific vocab tokens.
 
         """
+        if self.pre_process:
+            if not hasattr(self, 'prompt_table'):
+                raise AttributeError('Please set "use_soft_prompts" in the config to True')
 
-        if not hasattr(self, 'prompt_table'):
-            raise AttributeError('Please set "use_soft_prompts" in the config to True')
-
-        self.prompt_table.init_prompt_from_text(
-            prompt_tag,
-            init_token_ids,
-            word_embeddings=self.embedding.word_embeddings,
-            position_embeddings=self.embedding.position_embeddings,
-        )
+            self.prompt_table.init_prompt_from_text(
+                prompt_tag, prompt_id, init_token_ids, embeddings=self.embedding,
+            )
