@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import itertools
 import os
 import shutil
 import tempfile
@@ -25,18 +26,31 @@ from pytorch_lightning.overrides import LightningDistributedModule
 from pytorch_lightning.plugins.environments.cluster_environment import ClusterEnvironment
 from pytorch_lightning.plugins.io.checkpoint_plugin import CheckpointIO
 from pytorch_lightning.plugins.precision import NativeMixedPrecisionPlugin
+from pytorch_lightning.plugins.precision.native_amp import NativeMixedPrecisionPlugin
 from pytorch_lightning.plugins.training_type.ddp import DDPPlugin
+from pytorch_lightning.trainer.connectors.data_connector import DataConnector
+from pytorch_lightning.utilities.exceptions import MisconfigurationException
+from pytorch_lightning.utilities.fetching import (
+    AbstractDataFetcher,
+    DataFetcher,
+    DataLoaderIterDataFetcher,
+    InterBatchParallelDataFetcher,
+)
+from pytorch_lightning.utilities.signature_utils import is_param_in_hook_signature
 from pytorch_lightning.utilities.types import _PATH
+from pytorch_lightning.utilities.warnings import rank_zero_warn
 from torch.distributed.algorithms.ddp_comm_hooks.debugging_hooks import noop_hook
 from torch.nn.parallel import DistributedDataParallel
 
 from nemo.collections.nlp.modules.common.megatron.module import Float16Module
 from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
-from nemo.core.optim import MasterOptimizerWrapper
+from nemo.core.optim import MainParamsOptimizerWrapper
 from nemo.utils import AppState, logging
+from nemo.utils.model_utils import inject_model_parallel_rank
 
 try:
     from apex.transformer import parallel_state
+    from apex.transformer.pipeline_parallel.utils import get_num_microbatches
 
     HAVE_APEX = True
 
@@ -65,12 +79,12 @@ class NLPDDPPlugin(DDPPlugin):
         no_ddp_communication_hook: bool = False,
         **kwargs: Union[Any, Dict[str, Any]],
     ) -> None:
-        super().__init__(parallel_devices, num_nodes, cluster_environment, checkpoint_io, sync_batchnorm, **kwargs)
-
         if not HAVE_APEX:
-            logging.warning(
+            raise ImportError(
                 "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
             )
+        super().__init__(parallel_devices, num_nodes, cluster_environment, checkpoint_io, sync_batchnorm, **kwargs)
+
         self.no_ddp_communication_hook = no_ddp_communication_hook
 
     def setup_distributed(self, global_rank: int = None, world_size: int = None) -> None:
@@ -78,10 +92,11 @@ class NLPDDPPlugin(DDPPlugin):
         super().setup_distributed()
 
         # init model parallel if needed
-        app_state = AppState()
+        if parallel_state.is_unitialized():
+            app_state = AppState()
 
-        if app_state.model_parallel_size is not None:
-            self.init_model_parallel(app_state.global_rank, app_state.world_size)
+            if app_state.model_parallel_size is not None:
+                self.init_model_parallel(app_state.global_rank, app_state.world_size)
 
     def configure_ddp(self):
         """ Override LightningModule ddp if using model parallel.
@@ -96,21 +111,21 @@ class NLPDDPPlugin(DDPPlugin):
             # With model parallelism, multiple GPUs form a large "logical GPU"
             # this means that data parallel groups span multiple GPUs
             # and are non-trivial
-            device_ids = self.determine_ddp_device_ids()
+            # TODO: for megatron-lm self.model is a list
+            self.pre_configure_ddp()
+            # device_ids = self.determine_ddp_device_ids()
             self._model = DistributedDataParallel(
                 LightningDistributedModule(self.model),
-                device_ids=device_ids,
-                output_device=device_ids[0],
-                process_group=app_state.data_parallel_group,
-                find_unused_parameters=False,
+                process_group=parallel_state.get_data_parallel_group(),
                 **self._ddp_kwargs,
             )
-
+            self._register_ddp_hooks()
             if self.no_ddp_communication_hook:
                 # When using custom gradient accumulation and allreduce, disable
                 # DDP communication hook that works on the gradient bucket.
                 # Instead, use the custom gradient function and communication hook,
                 # which is defined in the master optimizer wrapper.
+                self._model.require_backward_grad_sync = False
                 self._model.register_comm_hook(None, noop_hook)
 
         else:
@@ -129,52 +144,65 @@ class NLPDDPPlugin(DDPPlugin):
         # we initialize megatron-lm model parallel and data parallel groups
         # after initializing DDP with PTL.
         if app_state.model_parallel_size is not None:
+            # destroy groups in case they have already been created
+            # this happens with multiple calls to trainer.test for example
+            parallel_state.destroy_model_parallel()
             if torch.distributed.is_initialized():
-                parallel_state.initialize_model_parallel(app_state.model_parallel_size)
-                app_state.model_parallel_group = parallel_state.get_tensor_model_parallel_group()
+                parallel_state.initialize_model_parallel(
+                    tensor_model_parallel_size_=app_state.tensor_model_parallel_size,
+                    pipeline_model_parallel_size_=app_state.pipeline_model_parallel_size,
+                )
+
+                # assert that fake tp and pp rank match after model parallel init
+                assert app_state.tensor_model_parallel_rank == parallel_state.get_tensor_model_parallel_rank()
+                assert app_state.pipeline_model_parallel_rank == parallel_state.get_pipeline_model_parallel_rank()
+
+                app_state.tensor_model_parallel_group = parallel_state.get_tensor_model_parallel_group()
                 app_state.data_parallel_group = parallel_state.get_data_parallel_group()
-                app_state.model_parallel_rank = parallel_state.get_tensor_model_parallel_rank()
                 app_state.data_parallel_rank = parallel_state.get_data_parallel_rank()
                 app_state.data_parallel_size = parallel_state.get_data_parallel_world_size()
-                logging.info(f'mp_rank: {app_state.model_parallel_rank}')
-                logging.info(f'dp_rank: {app_state.data_parallel_rank}')
+                app_state.pipeline_model_parallel_group = parallel_state.get_pipeline_model_parallel_group()
 
     def save_checkpoint(self, checkpoint: Dict[str, Any], filepath: _PATH) -> None:
         # PTL override to accomodate model parallel checkpoints
-        filepath = self._inject_model_parallel_rank(filepath)
+        filepath = inject_model_parallel_rank(filepath)
         return super().save_checkpoint(checkpoint, filepath)
 
     def load_model_state_dict(self, checkpoint: Mapping[str, Any]) -> None:
         # Release strict state dict matching when using Megatron AMP-O2 to skip matching
         # half-precision module wrapper module.
+        # TODO: Refactor this to be more generic.
+        model_key = None
+        model_attr = None
         if hasattr(self.lightning_module, 'model'):
-            if isinstance(self.lightning_module.model, Float16Module):
+            model_key = 'model'
+            model_attr = self.lightning_module.model
+        elif hasattr(self.lightning_module, 'enc_dec_model'):
+            model_key = 'enc_dec_model'
+            model_attr = self.lightning_module.enc_dec_model
+        if model_key is not None:
+            if isinstance(model_attr, Float16Module):
                 new_state_dict = {}
                 for key in checkpoint['state_dict'].keys():
-                    new_key = key.replace('model.', 'model.module.', 1)
+                    new_key = key.replace(f'{model_key}.', f'{model_key}.module.', 1)
                     new_state_dict[new_key] = checkpoint['state_dict'][key]
                 checkpoint['state_dict'] = new_state_dict
 
         self.lightning_module.load_state_dict(checkpoint["state_dict"])
 
+    def load_checkpoint(self, checkpoint_path: _PATH) -> Dict[str, Any]:
+        """ PTL override to accomodate model parallel checkpoints """
+        # TODO: move to CheckpointIO
+        torch.cuda.empty_cache()
+        checkpoint_path = inject_model_parallel_rank(checkpoint_path)
+        return self.checkpoint_io.load_checkpoint(checkpoint_path)
+
     def remove_checkpoint(self, filepath: _PATH) -> None:
         # PTL override to accomodate model parallel checkpoints
-        filepath = self._inject_model_parallel_rank(filepath)
+        filepath = inject_model_parallel_rank(filepath)
         logging.info(f'Removing checkpoint: {filepath}')
         return super().remove_checkpoint(filepath)
 
-    def _inject_model_parallel_rank(self, filepath):
-        app_state = AppState()
-        # inserts mp_rank_XX for model parallel checkpoints
-        if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
-            # filepath needs to be updated to include mp_rank
-            dirname = os.path.dirname(filepath)
-            basename = os.path.basename(filepath)
-            filepath = f'{dirname}/mp_rank_{app_state.model_parallel_rank:02d}/{basename}'
-            return filepath
-        else:
-            return filepath
-
     @property
     def should_rank_save_checkpoint(self) -> bool:
         # PTL override that determines if checkpoints should be saved based on rank
@@ -203,11 +231,15 @@ class NLPDDPPlugin(DDPPlugin):
 
 class NLPSaveRestoreConnector(SaveRestoreConnector):
     def __init__(self) -> None:
-        super().__init__()
         if not HAVE_APEX:
             logging.warning(
-                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/apex\n"
+                "Megatron-based models require Apex to function correctly."
             )
+            # raise ImportError(
+            #    "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+            # )
+        super().__init__()
 
     def save_to(self, model, save_path: str):
         app_state = AppState()
@@ -216,41 +248,94 @@ class NLPSaveRestoreConnector(SaveRestoreConnector):
             dir_name = os.path.dirname(save_path)
 
             # first we save the weights for each model parallel rank
-            if app_state.data_parallel_rank == 0:
-                mp_model_weights = os.path.join(
-                    dir_name, f'mp_rank_{app_state.model_parallel_rank:02d}_' + self.model_weights_ckpt
-                )
-                self._save_state_dict_to_disk(model.state_dict(), mp_model_weights)
-
-            if torch.distributed.is_initialized():
-                torch.distributed.barrier()
-
-            # create nemo file from folder with all mp_ranks checkpoints
-            if app_state.model_parallel_rank == 0 and app_state.data_parallel_rank == 0:
-                with tempfile.TemporaryDirectory() as tmpdir:
-
-                    # move weights to the tmpdir
-                    for mp_rank in range(app_state.model_parallel_size):
-                        os.makedirs(os.path.join(tmpdir, f'mp_rank_{mp_rank:02d}'))
-                        mp_model_weights = os.path.join(dir_name, f'mp_rank_{mp_rank:02d}_' + self.model_weights_ckpt)
-                        shutil.move(
-                            mp_model_weights, os.path.join(tmpdir, f'mp_rank_{mp_rank:02d}', self.model_weights_ckpt)
+            if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
+                if app_state.data_parallel_rank == 0:
+                    if app_state.pipeline_model_parallel_size == 1:
+                        mp_model_weights = os.path.join(
+                            dir_name, f'mp_rank_{app_state.tensor_model_parallel_rank:02d}_' + self.model_weights_ckpt
+                        )
+                    else:
+                        mp_model_weights = os.path.join(
+                            dir_name,
+                            f'tp_rank_{app_state.tensor_model_parallel_rank:02d}_pp_rank_{app_state.pipeline_model_parallel_rank:03d}_'
+                            + self.model_weights_ckpt,
                         )
 
-                    # create config and artifacts in tmpdir
-                    config_yaml = os.path.join(tmpdir, self.model_config_yaml)
-                    model.to_config_file(path2yaml_file=config_yaml)
-                    if hasattr(model, 'artifacts') and model.artifacts is not None:
-                        self._handle_artifacts(model, nemo_file_folder=tmpdir)
-                        self._update_artifact_paths(model, path2yaml_file=config_yaml)
+                    self._save_state_dict_to_disk(model.state_dict(), mp_model_weights)
 
-                    # create tar file
-                    self._make_nemo_file_from_folder(save_path, tmpdir)
+                if torch.distributed.is_initialized():
+                    torch.distributed.barrier()
+
+                # create nemo file from folder with all mp_ranks checkpoints
+                if (
+                    app_state.pipeline_model_parallel_rank == 0
+                    and app_state.tensor_model_parallel_rank == 0
+                    and app_state.data_parallel_rank == 0
+                ):
+                    with tempfile.TemporaryDirectory() as tmpdir:
+
+                        if app_state.pipeline_model_parallel_size == 1:
+                            # move weights to the tmpdir
+                            for tp_rank in range(app_state.tensor_model_parallel_size):
+                                os.makedirs(os.path.join(tmpdir, f'mp_rank_{tp_rank:02d}'))
+                                mp_model_weights = os.path.join(
+                                    dir_name, f'mp_rank_{tp_rank:02d}_' + self.model_weights_ckpt
+                                )
+                                shutil.move(
+                                    mp_model_weights,
+                                    os.path.join(tmpdir, f'mp_rank_{tp_rank:02d}', self.model_weights_ckpt),
+                                )
+                        else:
+                            # move weights to the tmpdir
+                            for tp_rank, pp_rank in itertools.product(
+                                range(app_state.tensor_model_parallel_size),
+                                range(app_state.pipeline_model_parallel_size),
+                            ):
+                                os.makedirs(os.path.join(tmpdir, f'tp_rank_{tp_rank:02d}_pp_rank_{pp_rank:03d}'))
+                                mp_model_weights = os.path.join(
+                                    dir_name, f'tp_rank_{tp_rank:02d}_pp_rank_{pp_rank:03d}_' + self.model_weights_ckpt
+                                )
+                                shutil.move(
+                                    mp_model_weights,
+                                    os.path.join(
+                                        tmpdir, f'tp_rank_{tp_rank:02d}_pp_rank_{pp_rank:03d}', self.model_weights_ckpt
+                                    ),
+                                )
+
+                        # create config and artifacts in tmpdir
+                        config_yaml = os.path.join(tmpdir, self.model_config_yaml)
+                        model.to_config_file(path2yaml_file=config_yaml)
+                        if hasattr(model, 'artifacts') and model.artifacts is not None:
+                            self._handle_artifacts(model, nemo_file_folder=tmpdir)
+                            self._update_artifact_paths(model, path2yaml_file=config_yaml)
+
+                        # create tar file
+                        self._make_nemo_file_from_folder(save_path, tmpdir)
 
         else:
             return super().save_to(model, save_path)
 
 
+class PipelineMixedPrecisionPlugin(NativeMixedPrecisionPlugin):
+    """ Overrides PTL autocasting to not wrap training/val/test_step.
+        We do this because we have the Apex fwd/bwd functions in training_step.
+        This means .backward is being called in training_step so we do not want the whole
+        step wrapped in autocast.
+
+        We instead wrap the fwd_output_and_loss_func that is passed to the Apex fwd/bwd functions.
+    """
+
+    def __init__(
+        self, precision: Union[str, int], device: str, scaler: Optional[torch.cuda.amp.GradScaler] = None
+    ) -> None:
+        super().__init__(precision, device, scaler=scaler)
+
+    @contextmanager
+    def forward_context(self) -> Generator[None, None, None]:
+        """Have the PTL context manager do nothing."""
+        yield
+
+
 class GradScaler(torch.cuda.amp.GradScaler):
     """
     Gradient sclaer for model-parallel inf check. The inf in gradients are checked across tensor-parallel
@@ -448,7 +533,7 @@ class MegatronHalfPrecisionPlugin(NativeMixedPrecisionPlugin):
         **kwargs: Any,
     ) -> None:
         assert isinstance(
-            optimizer, MasterOptimizerWrapper
+            optimizer, MainParamsOptimizerWrapper
         ), "MegatronHalfPrecisionPlugin supports only the optimizer with master parameters"
 
         if self.scaler is None:
@@ -459,7 +544,6 @@ class MegatronHalfPrecisionPlugin(NativeMixedPrecisionPlugin):
                     _ = closure()
             else:
                 _ = closure()
-                optimizer.allreduce_main_grads()
 
             self._after_closure(model, optimizer, optimizer_idx)
             return optimizer.step(**kwargs)
@@ -485,7 +569,6 @@ class MegatronHalfPrecisionPlugin(NativeMixedPrecisionPlugin):
             # note: the scaler will skip the `optimizer.step` if nonfinite gradients are found
             self.scaler.step(optimizer, **kwargs)
             self.scaler.update()
-            model.log('grad_scale', self.scaler.get_scale())
 
     @contextmanager
     def forward_context(self) -> Generator[None, None, None]:
@@ -494,3 +577,69 @@ class MegatronHalfPrecisionPlugin(NativeMixedPrecisionPlugin):
             yield
         finally:
             pass
+
+
+class NLPDataConnector(DataConnector):
+    """ Override PTL DataConnector. Used to select custom data fetcher."""
+
+    def __init__(
+        self,
+        trainer: "pl.Trainer",
+        multiple_trainloader_mode: str = "max_size_cycle",
+        train_data_fetcher: Optional[AbstractDataFetcher] = None,
+        validate_data_fetcher: Optional[AbstractDataFetcher] = None,
+        test_data_fetcher: Optional[AbstractDataFetcher] = None,
+    ):
+
+        if not HAVE_APEX:
+            logging.warning("Apex was not found. Using model parallel or megatron models will error out.")
+
+        super().__init__(
+            trainer,
+            multiple_trainloader_mode=multiple_trainloader_mode,
+            train_data_fetcher=train_data_fetcher,
+            validate_data_fetcher=validate_data_fetcher,
+            test_data_fetcher=test_data_fetcher,
+        )
+
+    def _select_data_fetcher(self) -> AbstractDataFetcher:
+        if self.trainer.sanity_checking:
+            return GlobalBatchDataFetcher()
+
+        training_step_fx = getattr(self.trainer.lightning_module, "training_step")
+        if self.trainer.training and is_param_in_hook_signature(training_step_fx, "dataloader_iter", explicit=True):
+            rank_zero_warn(
+                "Found `dataloader_iter` argument in the `training_step`. Note that the support for "
+                "this signature is experimental and the behavior is subject to change."
+            )
+            return DataLoaderIterDataFetcher()
+
+        elif self.trainer.training and os.getenv("PL_INTER_BATCH_PARALLELISM", "0") == "1":
+            # note: this is an experimental feature
+            if not self.trainer.training_type_plugin.on_gpu:
+                raise MisconfigurationException("Inter batch parallelism is available only when using Nvidia GPUs.")
+            return InterBatchParallelDataFetcher()
+
+        return GlobalBatchDataFetcher()
+
+
+class GlobalBatchDataFetcher(DataFetcher):
+    """ Overrides PTL DataFetcher. Used to fetch global batches."""
+
+    def __init__(self, prefetch_batches: int = 0, store_on_device: bool = False) -> None:
+
+        if not HAVE_APEX:
+            logging.warning("Apex was not found. Using model parallel or megatron models will error out.")
+
+        super().__init__(prefetch_batches=prefetch_batches, store_on_device=store_on_device)
+        self.num_micro_batches = get_num_microbatches()
+
+    def _fetch_next_batch(self):
+        """ Fetches the next global batch which is a list of micro batches"""
+        with self.apply_profiler(f"get_{self.stage}_batch"):
+            with self.fetching_context():
+                data = self.on_fetch_start()
+                with self.apply_profiler(f"fetch_next_{self.stage}_batch"):
+                    batch = [next(self.dataloader_iter) for _ in range(self.num_micro_batches)]
+                self.fetched += 1
+                self.on_fetch_end(batch, data)
