@@ -30,6 +30,7 @@ from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.core.neural_types.elements import *
 from nemo.core.neural_types.neural_type import NeuralType
 from nemo.utils import logging
+from nemo.utils.decorators import deprecated
 
 
 @dataclass
@@ -42,6 +43,7 @@ class FastSpeech2Config:
     validation_ds: Optional[Dict[Any, Any]] = None
 
 
+@deprecated(version="1.8", explanation="FastSpeech2Model will be removed. Please, use FastPitchModel instead.")
 class FastSpeech2Model(SpectrogramGenerator):
     """FastSpeech 2 model used to convert from text (phonemes) to mel-spectrograms."""
 
@@ -98,7 +100,7 @@ class FastSpeech2Model(SpectrogramGenerator):
         },
         output_types={
             "mel_spec": NeuralType(('B', 'T', 'C'), MelSpectrogramType()),
-            "log_dur_preds": NeuralType(('B', 'T'), TokenDurationType(), optional=True),
+            "log_dur_preds": NeuralType(('B', 'T'), TokenLogDurationType(), optional=True),
             "pitch_preds": NeuralType(('B', 'T'), RegressionValuesType(), optional=True),
             "energy_preds": NeuralType(('B', 'T'), RegressionValuesType(), optional=True),
             "encoded_text_mask": NeuralType(('B', 'T', 'D'), MaskType()),
