@@ -27,6 +27,7 @@ from omegaconf import DictConfig, OmegaConf, open_dict
 from pytorch_lightning import LightningModule, Trainer
 from pytorch_lightning.utilities import rank_zero_only
 
+from nemo import package_info
 from nemo.core import optim
 from nemo.core.classes.common import Model
 from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
@@ -35,6 +36,13 @@ from nemo.utils import logging, model_utils
 from nemo.utils.app_state import AppState
 from nemo.utils.get_rank import is_global_rank_zero
 
+try:
+    from nemo.collections.nlp.parts.nlp_overrides import NLPDDPPlugin
+
+    HAVE_NLPPLUGIN = True
+except (ImportError, ModuleNotFoundError):
+    HAVE_NLPPLUGIN = False
+
 __all__ = ['ModelPT']
 
 
@@ -102,12 +110,17 @@ class ModelPT(LightningModule, Model):
             cfg.target = "{0}.{1}".format(self.__class__.__module__, self.__class__.__name__)
             OmegaConf.set_struct(cfg, True)
 
+        if 'nemo_version' not in cfg:
+            with open_dict(cfg):
+                cfg.nemo_version = package_info.__version__
+
         self._cfg = cfg
 
         self.save_hyperparameters("cfg")
         self._train_dl = None
         self._validation_dl = None
         self._test_dl = None
+        self._optimizer_param_groups = None
         self._optimizer = None
         self._scheduler = None
         self.trainer = trainer  # reference required for self.*_rank
@@ -125,10 +138,10 @@ class ModelPT(LightningModule, Model):
                 self.setup_training_data(self._cfg.train_ds)
 
             if 'validation_ds' in self._cfg and self._cfg.validation_ds is not None:
-                self.setup_multiple_validation_data(val_data_config=None)
+                self.setup_multiple_validation_data(val_data_config=cfg.validation_ds)
 
             if 'test_ds' in self._cfg and self._cfg.test_ds is not None:
-                self.setup_multiple_test_data(test_data_config=None)
+                self.setup_multiple_test_data(test_data_config=cfg.test_ds)
 
         else:
             if 'train_ds' in self._cfg and self._cfg.train_ds is not None:
@@ -426,6 +439,10 @@ class ModelPT(LightningModule, Model):
                 The list of "arg_value" will be parsed and a dictionary of optimizer kwargs \
                 will be built and supplied to instantiate the optimizer.
         """
+
+        if self._optimizer_param_groups is None:
+            self.setup_optimizer_param_groups()
+
         # If config was not explicitly passed to us
         if optim_config is None:
             # See if internal config has `optim` namespace
@@ -458,6 +475,7 @@ class ModelPT(LightningModule, Model):
             logging.warning(f"Trainer wasn't specified in model constructor. Make sure that you really wanted it.")
 
         if 'sched' in optim_config and self._trainer is not None:
+
             if not isinstance(self._trainer.accumulate_grad_batches, int):
                 raise ValueError("We do not currently support gradient acculumation that is not an integer.")
             if self._trainer.max_steps is None or self.trainer.max_steps < 0:
@@ -471,6 +489,9 @@ class ModelPT(LightningModule, Model):
                     optim_config['sched']['t_num_workers'] = self._trainer.num_processes * self._trainer.num_nodes
                 elif self._trainer.accelerator == "ddp":
                     optim_config['sched']['t_num_workers'] = self._trainer.num_gpus * self._trainer.num_nodes
+                elif HAVE_NLPPLUGIN and isinstance(self._trainer.accelerator.training_type_plugin, NLPDDPPlugin):
+                    app = AppState()
+                    optim_config['sched']['t_num_workers'] = app.data_parallel_size
                 else:
                     logging.warning(
                         f"The lightning trainer received accelerator: {self._trainer.accelerator}. We "
@@ -528,7 +549,7 @@ class ModelPT(LightningModule, Model):
         # Actually instantiate the optimizer
         if optimizer_cls is not None:
             if inspect.isclass(optimizer_cls):
-                optimizer = optimizer_cls(self.parameters(), **optimizer_args)
+                optimizer = optimizer_cls(self._optimizer_param_groups, **optimizer_args)
                 logging.info("Optimizer config = %s", str(optimizer))
 
                 self._optimizer = optimizer
@@ -544,7 +565,7 @@ class ModelPT(LightningModule, Model):
                     optimizer_config.update(optimizer_args)
 
                     optimizer_instance = hydra.utils.instantiate(
-                        optimizer_cls, self.parameters(), **optimizer_config
+                        optimizer_cls, self._optimizer_param_groups, **optimizer_config
                     )  # type: DictConfig
 
                     logging.info("Optimizer config = %s", str(optimizer_instance))
@@ -561,7 +582,7 @@ class ModelPT(LightningModule, Model):
 
         else:
             optimizer = optim.get_optimizer(optimizer_name)
-            optimizer = optimizer(self.parameters(), **optimizer_args)
+            optimizer = optimizer(self._optimizer_param_groups, **optimizer_args)
 
             logging.info("Optimizer config = %s", str(optimizer))
 
@@ -576,6 +597,23 @@ class ModelPT(LightningModule, Model):
         # This return allows multiple optimizers or schedulers to be created
         return self._optimizer, self._scheduler
 
+    def setup_optimizer_param_groups(self):
+        """
+            Used to create param groups for the optimizer.
+            As an example, this can be used to specify per-layer learning rates:
+            optim.SGD([
+                        {'params': model.base.parameters()},
+                        {'params': model.classifier.parameters(), 'lr': 1e-3}
+                        ], lr=1e-2, momentum=0.9)
+            See https://pytorch.org/docs/stable/optim.html for more information.
+            By default, ModelPT will use self.parameters().
+            Override this method to add custom param groups.
+    """
+        param_groups = None
+        if hasattr(self, 'parameters'):
+            param_groups = [{'params': self.parameters()}]
+        self._optimizer_param_groups = param_groups
+
     def configure_optimizers(self):
         self.setup_optimization()
 
@@ -856,6 +894,36 @@ class ModelPT(LightningModule, Model):
         """
         return self._test_names[dataloader_idx]
 
+    def load_part_of_state_dict(self, state_dict, include, exclude, load_from_string):
+
+        excluded_param_names = []
+        # create dict
+        dict_to_load = {}
+        for k, v in state_dict.items():
+            should_add = False
+            # if any string in include is present, should add
+            for p in include:
+                if p in k:
+                    should_add = True
+                    break
+            # except for if any string from exclude is present
+            for e in exclude:
+                if e in k:
+                    excluded_param_names.append(k)
+                    should_add = False
+                    break
+            if should_add:
+                dict_to_load[k] = v
+
+        # Restore checkpoint part into current model
+        self.load_state_dict(dict_to_load, strict=False)
+        logging.info(f'Model checkpoint partially restored from {load_from_string}')
+        if len(excluded_param_names) > 0:
+            logging.info(
+                f'The following parameters were excluded from loading from {load_from_string} : {excluded_param_names}'
+            )
+            logging.info(f'Make sure that this is what you wanted!')
+
     @rank_zero_only
     def maybe_init_from_pretrained_checkpoint(self, cfg: OmegaConf, map_location: str = 'cpu'):
         """
@@ -864,15 +932,36 @@ class ModelPT(LightningModule, Model):
         requirement of exact model parameters matching.
 
         Initializations:
-            init_from_nemo_model: Str path to a .nemo model, which will be instantiated in order
-                to extract the state dict.
+            init_from_nemo_model: Str path to a .nemo model in order to load state_dict from single nemo file;
+            if loading from multiple files, pass in a dict where the values have the following fields:
+
+                path: Str path to .nemo model
+
+                include: Optional list of strings, at least one of which needs to be contained in parameter name
+                to be loaded from this .nemo file. Default: everything is included.
+
+                exclude: Optional list of strings, which can be used to exclude any parameter containing one of
+                these strings from being loaded from this .nemo file. Default: nothing is excluded.
+
+                hydra usage example:
+
+                init_from_nemo_model:
+                    model0:
+                        path:<path/to/model1>
+                        include:["encoder"]
+                    model1:
+                        path:<path/to/model2>
+                        include:["decoder"]
+                        exclude:["embed"]
 
             init_from_pretrained_model: Str name of a pretrained model checkpoint (obtained via cloud).
                 The model will be downloaded (or a cached copy will be used), instantiated and then
-                its state dict will be extracted.
+                its state dict will be extracted. If loading from multiple models, you can pass in a dict
+                with the same format as for init_from_nemo_model, except with "name" instead of "path"
 
             init_from_ptl_ckpt: Str name of a Pytorch Lightning checkpoint file. It will be loaded and
-                the state dict will extracted.
+                the state dict will extracted. If loading from multiple files, you can pass in a dict
+                with the same format as for init_from_nemo_model.
 
         Args:
             cfg: The config used to instantiate the model. It need only contain one of the above keys.
@@ -880,7 +969,11 @@ class ModelPT(LightningModule, Model):
                 (from the pretrained model or checkpoint) will be loaded.
 
         """
-        args = ['init_from_nemo_model', 'init_from_pretrained_model', 'init_from_ptl_ckpt']
+        args = [
+            'init_from_nemo_model',
+            'init_from_pretrained_model',
+            'init_from_ptl_ckpt',
+        ]
         arg_matches = [(1 if arg in cfg and arg is not None else 0) for arg in args]
 
         if sum(arg_matches) == 0:
@@ -895,57 +988,119 @@ class ModelPT(LightningModule, Model):
 
         if 'init_from_nemo_model' in cfg and cfg.init_from_nemo_model is not None:
             with open_dict(cfg):
-                # Restore model
-                model_path = cfg.pop('init_from_nemo_model')
-                restored_model = self.restore_from(
-                    model_path, map_location=map_location, strict=cfg.get("init_strict", True)
-                )
+                if isinstance(cfg.init_from_nemo_model, str):
+                    model_path = cfg.init_from_nemo_model
+                    # Restore model
+                    restored_model = self.restore_from(
+                        model_path, map_location=map_location, strict=cfg.get("init_strict", True)
+                    )
+                    # Restore checkpoint into current model
+                    self.load_state_dict(restored_model.state_dict(), strict=False)
+                    logging.info(f'Model checkpoint restored from nemo file with path : `{model_path}`')
+                    del restored_model
+                elif isinstance(cfg.init_from_nemo_model, (DictConfig, dict)):
+                    model_load_dict = cfg.init_from_nemo_model
+                    for model_load_cfg in model_load_dict.values():
+                        model_path = model_load_cfg.path
+                        # Restore model
+                        restored_model = self.restore_from(
+                            model_path, map_location=map_location, strict=cfg.get("init_strict", True)
+                        )
 
-                # Restore checkpoint into current model
-                self.load_state_dict(restored_model.state_dict(), strict=False)
-                logging.info(f'Model checkpoint restored from nemo file with path : `{model_path}`')
+                        include = model_load_cfg.pop('include', [""])
+                        exclude = model_load_cfg.pop('exclude', [])
 
-                del restored_model
+                        self.load_part_of_state_dict(
+                            restored_model.state_dict(), include, exclude, f'nemo file with path `{model_path}`'
+                        )
+
+                        del restored_model
+                else:
+                    raise TypeError("Invalid type: init_from_nemo_model is not a string or a dict!")
 
         if 'init_from_pretrained_model' in cfg and cfg.init_from_pretrained_model is not None:
             with open_dict(cfg):
                 # Restore model
-                model_name = cfg.pop('init_from_pretrained_model')
 
-                # Check if model is being resumed or not - only works if `Trainer` is attached to model
-                if hasattr(self, 'trainer') and self.trainer is not None:
-                    trainer = self.trainer
-                    if (
-                        hasattr(trainer, 'resume_from_checkpoint')
-                        and trainer.checkpoint_connector.resume_checkpoint_path is not None
-                    ):
-                        logging.info(
-                            "Model training is being resumed via Pytorch Lightning.\n"
-                            "Initialization from pretrained model (via cloud) will be skipped."
+                if isinstance(cfg.init_from_pretrained_model, str):
+                    model_name = cfg.pop('init_from_pretrained_model')
+
+                    # Check if model is being resumed or not - only works if `Trainer` is attached to model
+                    if hasattr(self, 'trainer') and self.trainer is not None:
+                        trainer = self.trainer
+                        if (
+                            hasattr(trainer, 'resume_from_checkpoint')
+                            and trainer.checkpoint_connector.resume_checkpoint_path is not None
+                        ):
+                            logging.info(
+                                "Model training is being resumed via Pytorch Lightning.\n"
+                                "Initialization from pretrained model (via cloud) will be skipped."
+                            )
+                            return
+
+                    restored_model = self.from_pretrained(
+                        model_name, map_location=map_location, strict=cfg.get("init_strict", True)
+                    )
+
+                    # Restore checkpoint into current model
+                    self.load_state_dict(restored_model.state_dict(), strict=False)
+                    logging.info(f'Model checkpoint restored from pretrained chackpoint with name : `{model_name}`')
+
+                    del restored_model
+                elif isinstance(cfg.init_from_pretrained_model, (DictConfig, dict)):
+                    model_load_dict = cfg.init_from_pretrained_model
+                    for model_load_cfg in model_load_dict.values():
+                        model_name = model_load_cfg.name
+                        # Restore model
+                        restored_model = self.from_pretrained(
+                            model_name, map_location=map_location, strict=cfg.get("init_strict", True)
                         )
-                        return
 
-                restored_model = self.from_pretrained(
-                    model_name, map_location=map_location, strict=cfg.get("init_strict", True)
-                )
+                        include = model_load_cfg.pop('include', [""])
+                        exclude = model_load_cfg.pop('exclude', [])
 
-                # Restore checkpoint into current model
-                self.load_state_dict(restored_model.state_dict(), strict=False)
-                logging.info(f'Model checkpoint restored from pretrained chackpoint with name : `{model_name}`')
+                        self.load_part_of_state_dict(
+                            restored_model.state_dict(),
+                            include,
+                            exclude,
+                            f'pretrained chackpoint with name `{model_name}`',
+                        )
 
-                del restored_model
+                        del restored_model
+                else:
+                    raise TypeError("Invalid type: init_from_pretrained_model is not a string or a dict!")
 
         if 'init_from_ptl_ckpt' in cfg and cfg.init_from_ptl_ckpt is not None:
             with open_dict(cfg):
-                # Restore checkpoint
-                ckpt_path = cfg.pop('init_from_ptl_ckpt')
-                ckpt = torch.load(ckpt_path, map_location=map_location)
+                if isinstance(cfg.init_from_ptl_ckpt, str):
+                    # Restore checkpoint
+                    ckpt_path = cfg.pop('init_from_ptl_ckpt')
+                    ckpt = torch.load(ckpt_path, map_location=map_location)
 
-                # Restore checkpoint into current model
-                self.load_state_dict(ckpt['state_dict'], strict=False)
-                logging.info(f'Model checkpoint restored from pytorch lightning chackpoint with path : `{ckpt_path}`')
+                    # Restore checkpoint into current model
+                    self.load_state_dict(ckpt['state_dict'], strict=False)
+                    logging.info(
+                        f'Model checkpoint restored from pytorch lightning chackpoint with path : `{ckpt_path}`'
+                    )
 
-                del ckpt
+                    del ckpt
+                elif isinstance(cfg.init_from_ptl_ckpt, (DictConfig, dict)):
+                    model_load_dict = cfg.init_from_ptl_ckpt
+                    for model_load_cfg in model_load_dict.values():
+                        ckpt_path = model_load_cfg.path
+                        # Restore model
+                        ckpt = torch.load(ckpt_path, map_location=map_location)
+
+                        include = model_load_cfg.pop('include', [""])
+                        exclude = model_load_cfg.pop('exclude', [])
+
+                        self.load_part_of_state_dict(
+                            ckpt['state_dict'], include, exclude, f'nemo file with path `{model_path}`'
+                        )
+
+                        del ckpt
+                else:
+                    raise TypeError("Invalid type: init_from_ptl_ckpt is not a string or a dict!")
 
     def teardown(self, stage: str):
         """
