@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
+from enum import Enum
 from typing import Callable, Dict, Optional, Type
 
 import onnx
@@ -20,6 +22,126 @@ import torch.nn as nn
 
 from nemo.utils import logging
 
+try:
+    import onnxruntime
+
+    ort_available = True
+except (ImportError, ModuleNotFoundError):
+    ort_available = False
+
+
+class ExportFormat(Enum):
+    """Which format to use when exporting a Neural Module for deployment"""
+
+    ONNX = (1,)
+    TORCHSCRIPT = (2,)
+
+
+_EXT_DICT = {
+    ".pt": ExportFormat.TORCHSCRIPT,
+    ".ts": ExportFormat.TORCHSCRIPT,
+    ".onnx": ExportFormat.ONNX,
+}
+
+
+class CastToFloat(nn.Module):
+    def __init__(self, mod):
+        super(CastToFloat, self).__init__()
+        self.mod = mod
+
+    def forward(self, x):
+        if torch.is_autocast_enabled():
+            ret = self.mod.forward(x.to(torch.float)).to(x.dtype)
+        else:
+            ret = self.mod.forward(x)
+        return ret
+
+
+def get_export_format(filename: str):
+    _, ext = os.path.splitext(filename)
+    try:
+        return _EXT_DICT[ext]
+    except KeyError:
+        raise ValueError(f"Export file {filename} extension does not correspond to any export format!")
+
+
+def augment_filename(output: str, prepend: str):
+    path, filename = os.path.split(output)
+    filename = f"{prepend}-{filename}"
+    return os.path.join(path, filename)
+
+
+def forward_method(self):
+    if hasattr(self, "forward_for_export"):
+        return self.forward_for_export
+    else:
+        return self.forward
+
+
+def wrap_forward_method(self):
+    tp = type(self)
+    old_forward_method = None
+    if hasattr(tp, "forward_for_export"):
+        forward_method = tp.forward_for_export
+        old_forward_method = tp.forward
+        tp.forward = forward_method
+    else:
+        forward_method = None
+    return forward_method, old_forward_method
+
+
+def parse_input_example(input_example):
+    input_list = list(input_example)
+    input_dict = {}
+    # process possible kwargs
+    if isinstance(input_list[-1], dict):
+        input_dict = input_list[-1]
+        input_list = input_list[:-1]
+    return input_list, input_dict
+
+
+def to_onnxrt_input(input_names, input_list, input_dict):
+    odict = {}
+    for k, v in input_dict.items():
+        odict[k] = v.cpu().numpy()
+    for i, input in enumerate(input_list):
+        if type(input) in (list, tuple):
+            odict[input_names[i]] = tuple([ip.cpu().numpy() for ip in input])
+        else:
+            odict[input_names[i]] = input.cpu().numpy()
+    return odict
+
+
+def verify_runtime(
+    output, input_list, input_dict, input_names, output_names, output_example, check_tolerance=0.01,
+):
+    # Verify the model can be read, and is valid
+    onnx_model = onnx.load(output)
+    global ort_available
+    if not ort_available:
+        logging.warning(f"ONNX generated at {output}, not verified - please install onnxruntime_gpu package.\n")
+        onnx.checker.check_model(onnx_model, full_check=True)
+        return
+
+    onnx_session_opt = onnxruntime.SessionOptions()
+    onnx_session_opt.graph_optimization_level = onnxruntime.GraphOptimizationLevel.ORT_ENABLE_ALL
+    sess = onnxruntime.InferenceSession(
+        onnx_model.SerializeToString(), sess_options=onnx_session_opt, providers=['CUDAExecutionProvider']
+    )
+    ort_out = sess.run(output_names, to_onnxrt_input(input_names, input_list, input_dict))
+    all_good = True
+
+    for i, out in enumerate(ort_out[0]):
+        expected = output_example[i]
+        if torch.is_tensor(expected):
+            tout = torch.from_numpy(out)
+            if not torch.allclose(tout, expected.cpu(), rtol=check_tolerance, atol=100 * check_tolerance):
+                all_good = False
+                logging.info(f"onnxruntime results mismatch! PyTorch(expected):\n{expected}\nONNXruntime:\n{tout}")
+    status = "SUCCESS" if all_good else "FAIL"
+    logging.info(f"ONNX generated at {output} verified with onnxruntime : " + status)
+
+
 apex_available = True
 
 try:
@@ -50,94 +172,6 @@ except Exception as e:
     apex_available = False
 
 
-def expand_Conv1D(conv1d: nn.Module) -> Optional[nn.Conv2d]:
-    """
-    Expands a Conv1D into a Conv2D. This is required for many (closed source) commercial tools with poor support for 1D Convolutions in Onnx.
-    Args:
-        conv1d: the Conv1D pytorch module to expand
-    Returns:
-        conv2d: Conv2D module with identical weights and params
-    """
-    if not isinstance(conv1d, nn.Conv1d):
-        return None
-    conv2d = nn.Conv2d(
-        conv1d.in_channels,
-        conv1d.out_channels,
-        kernel_size=(conv1d.kernel_size[0], 1),
-        stride=(conv1d.stride[0], 1),
-        padding=(conv1d.padding[0], 0),
-        dilation=(conv1d.dilation[0], 1),
-        groups=conv1d.groups,
-        padding_mode=conv1d.padding_mode,
-    ).to(device=conv1d.weight.device, dtype=conv1d.weight.dtype)
-    conv2d.bias = conv1d.bias
-    conv2d.weight = nn.Parameter(conv1d.weight.unsqueeze(-1))
-    # check that expansion is valid
-    for _ in range(2):
-        sample_input = torch.rand(1, conv1d.in_channels, 256).to(
-            device=conv1d.weight.device, dtype=conv1d.weight.dtype
-        )
-        close = conv1d(sample_input).mean() - conv2d(sample_input.unsqueeze(-1)).squeeze().mean()
-        if close.abs() > 1.0:
-            raise ValueError("Unable to expand Conv1D to Conv2D")
-    return conv2d
-
-
-def expand_BatchNorm1d(bn1d: nn.Module) -> Optional[nn.BatchNorm2d]:
-    """
-    Expands a BatchNorm1d into a BatchNorm2d. This is required for many (closed source) commercial tools with poor support for BatchNorm1d in Onnx.
-    Args:
-        bn1d: the BatchNorm1d pytorch module to expand
-    Returns:
-        bn2d: BatchNorm2d module with identical weights and params
-    """
-    if not isinstance(bn1d, nn.BatchNorm1d):
-        return None
-    mod = torch.nn.BatchNorm2d(
-        bn1d.num_features,
-        eps=bn1d.eps,
-        momentum=bn1d.momentum,
-        affine=bn1d.affine,
-        track_running_stats=bn1d.track_running_stats,
-    ).to(device=conv1d.weight.device, dtype=conv1d.weight.dtype)
-    bn_state = bn1d.state_dict()
-    mod.load_state_dict(bn_state)
-    return mod
-
-
-def expand_ConvTranspose1D(conv1d: nn.Module) -> Optional[nn.ConvTranspose2d]:
-    """
-    Expands a Conv1D into a Conv2D. This is required for many (closed source) commercial tools with poor support for 1D Convolutions in Onnx.
-    Args:
-        conv1d: the Conv1D pytorch module to expand
-    Returns:
-        conv2d: Conv2D module with identical weights and params
-    """
-    if not isinstance(conv1d, nn.ConvTranspose1d):
-        return None
-    conv2d = nn.ConvTranspose2d(
-        conv1d.in_channels,
-        conv1d.out_channels,
-        kernel_size=(conv1d.kernel_size[0], 1),
-        stride=(conv1d.stride[0], 1),
-        padding=(conv1d.padding[0], 0),
-        dilation=(conv1d.dilation[0], 1),
-        groups=conv1d.groups,
-        padding_mode=conv1d.padding_mode,
-    ).to(device=conv1d.weight.device, dtype=conv1d.weight.dtype)
-    conv2d.bias = conv1d.bias
-    conv2d.weight = nn.Parameter(conv1d.weight.unsqueeze(-1))
-    # check that expansion is valid
-    for _ in range(2):
-        sample_input = torch.rand(1, conv1d.in_channels, 256).to(
-            device=conv1d.weight.device, dtype=conv1d.weight.dtype
-        )
-        close = conv1d(sample_input).mean() - conv2d(sample_input.unsqueeze(-1)).squeeze().mean()
-        if close.abs() > 1.0:
-            raise ValueError("Unable to expand Conv1D to Conv2D")
-    return conv2d
-
-
 def simple_replace(BaseT: Type[nn.Module], DestT: Type[nn.Module]) -> Callable[[nn.Module], Optional[nn.Module]]:
     """
     Generic function generator to replace BaseT module with DestT. BaseT and DestT should have same atrributes. No weights are copied.
@@ -158,6 +192,23 @@ def simple_replace(BaseT: Type[nn.Module], DestT: Type[nn.Module]) -> Callable[[
     return expansion_fn
 
 
+def wrap_module(BaseT: Type[nn.Module], DestT: Type[nn.Module]) -> Callable[[nn.Module], Optional[nn.Module]]:
+    """
+    Generic function generator to replace BaseT module with DestT wrapper. 
+    Args:
+        BaseT : module type to replace
+        DestT : destination module type
+    Returns:
+        swap function to replace BaseT module with DestT
+    """
+
+    def expansion_fn(mod: nn.Module) -> Optional[nn.Module]:
+        out = DestT(mod)
+        return out
+
+    return expansion_fn
+
+
 def swap_modules(model: nn.Module, mapping: Dict[str, nn.Module]):
     """
     This function swaps nested modules as specified by "dot paths" in mod with a desired replacement. This allows
@@ -195,21 +246,20 @@ def replace_modules(
             swapped = expansions[m_type](m)
             if swapped:
                 mapping[name] = swapped
-    logging.warning(f"Swapped {len(mapping)} modules")
+    if len(mapping) > 0:
+        logging.info(f"Swapped {len(mapping)} modules")
     swap_modules(model, mapping)
     return model
 
 
-default_1D_2D_replacements = {
-    "Conv1d": expand_Conv1D,
-    "ConvTranspose1d": expand_ConvTranspose1D,
-    "BatchNorm1d": expand_BatchNorm1d,
-    "AdaptiveAvgPool1d": simple_replace(nn.AdaptiveAvgPool1d, nn.AdaptiveAvgPool2d),
-    "AvgPool1d": simple_replace(nn.AvgPool1d, nn.AvgPool2d),
+default_replacements = {
+    "BatchNorm1d": wrap_module(nn.BatchNorm1d, CastToFloat),
+    "BatchNorm2d": wrap_module(nn.BatchNorm2d, CastToFloat),
+    "LayerNorm": wrap_module(nn.LayerNorm, CastToFloat),
 }
 
 
-def replace_for_export(model: nn.Module, replace_1D_2D: bool = False) -> nn.Module:
+def replace_for_export(model: nn.Module) -> nn.Module:
     """
     Top-level function to replace default set of modules in model
     NOTE: This occurs in place, if you want to preserve model then make sure to copy it first.
@@ -220,6 +270,4 @@ def replace_for_export(model: nn.Module, replace_1D_2D: bool = False) -> nn.Modu
         model, possibly modified in-place
     """
     replace_modules(model, default_Apex_replacements)
-    if replace_1D_2D:
-        # TODO: add squeeze/unsqueeze
-        replace_modules(model, default_1D_2D_replacements)
+    replace_modules(model, default_replacements)
